@@ -17,8 +17,7 @@ import hashlib
 import json
 from oslo_utils import encodeutils
 from requests import codes
-import six
-from six.moves.urllib import parse
+import urllib.parse
 import warlock
 
 from glanceclient.common import utils
@@ -55,7 +54,7 @@ class Controller(object):
 
     @staticmethod
     def _wrap(value):
-        if isinstance(value, six.string_types):
+        if isinstance(value, str):
             return [value]
         return value
 
@@ -142,19 +141,19 @@ class Controller(object):
         tags_url_params = []
 
         for tag in tags:
-            if not isinstance(tag, six.string_types):
+            if not isinstance(tag, str):
                 raise exc.HTTPBadRequest("Invalid tag value %s" % tag)
 
             tags_url_params.append({'tag': encodeutils.safe_encode(tag)})
 
         for param, value in filters.items():
-            if isinstance(value, six.string_types):
+            if isinstance(value, str):
                 filters[param] = encodeutils.safe_encode(value)
 
-        url = '/v2/images?%s' % parse.urlencode(filters)
+        url = '/v2/images?%s' % urllib.parse.urlencode(filters)
 
         for param in tags_url_params:
-            url = '%s&%s' % (url, parse.urlencode(param))
+            url = '%s&%s' % (url, urllib.parse.urlencode(param))
 
         if 'sort' in kwargs:
             if 'sort_key' in kwargs or 'sort_dir' in kwargs:
@@ -178,7 +177,7 @@ class Controller(object):
             for dir in sort_dir:
                 url = '%s&sort_dir=%s' % (url, dir)
 
-        if isinstance(kwargs.get('marker'), six.string_types):
+        if isinstance(kwargs.get('marker'), str):
             url = '%s&marker=%s' % (url, kwargs['marker'])
 
         for image, resp in paginate(url, page_size, limit):
@@ -197,6 +196,25 @@ class Controller(object):
     def get(self, image_id):
         return self._get(image_id)
 
+    @utils.add_req_id_to_object()
+    def get_associated_image_tasks(self, image_id):
+        """Get the tasks associated with an image.
+
+        :param image_id: ID of the image
+        :raises: exc.HTTPNotImplemented if Glance is not new enough to support
+                 this API (v2.12).
+        """
+        # NOTE (abhishekk): Verify that /v2i/images/%s/tasks is supported by
+        # glance
+        if utils.has_version(self.http_client, 'v2.12'):
+            url = '/v2/images/%s/tasks' % image_id
+            resp, body = self.http_client.get(url)
+            body.pop('self', None)
+            return body, resp
+        else:
+            raise exc.HTTPNotImplemented(
+                'This operation is not supported by Glance.')
+
     @utils.add_req_id_to_object()
     def data(self, image_id, do_checksum=True, allow_md5_fallback=False):
         """Retrieve data of an image.
@@ -209,9 +227,11 @@ class Controller(object):
            specified hash algorithm is not available AND allow_md5_fallback
            is True, then continue to step #2
         2. else if the image has a checksum property, MD5 is used to
-           validate against the 'checksum' value
+           validate against the 'checksum' value.  (If MD5 is not available
+           to the client, the download fails.)
         3. else if the download response has a 'content-md5' header, MD5
-           is used to validate against the header value
+           is used to validate against the header value.  (If MD5 is not
+           available to the client, the download fails.)
         4. if none of 1-3 obtain, the data is **not validated** (this is
            compatible with legacy behavior)
 
@@ -302,6 +322,14 @@ class Controller(object):
         resp, body = self.http_client.get(url)
         return body, resp
 
+    @utils.add_req_id_to_object()
+    def delete_from_store(self, store_id, image_id):
+        """Delete image data from specific store."""
+        url = ('/v2/stores/%(store)s/%(image)s' % {'store': store_id,
+                                                   'image': image_id})
+        resp, body = self.http_client.delete(url)
+        return body, resp
+
     @utils.add_req_id_to_object()
     def stage(self, image_id, image_data, image_size=None):
         """Upload the data to image staging.
@@ -318,13 +346,22 @@ class Controller(object):
 
     @utils.add_req_id_to_object()
     def image_import(self, image_id, method='glance-direct', uri=None,
-                     backend=None):
+                     backend=None, stores=None, allow_failure=True,
+                     all_stores=None):
         """Import Image via method."""
         headers = {}
         url = '/v2/images/%s/import' % image_id
         data = {'method': {'name': method}}
+        if stores:
+            data['stores'] = stores
+            if allow_failure:
+                data['all_stores_must_succeed'] = False
         if backend is not None:
             headers['x-image-meta-store'] = backend
+        if all_stores:
+            data['all_stores'] = True
+            if allow_failure:
+                data['all_stores_must_succeed'] = False
 
         if uri:
             if method == 'web-download':
